"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const builder_utils_1 = require("@ewizardjs/builder-utils");
const rules_1 = require("@ewizardjs/builder-utils/rules");
const settings_1 = require("@ewizardjs/settings");
const system_settings_1 = require("@ewizardjs/system-settings");
const fs_extra_1 = require("fs-extra");
const globby_1 = __importDefault(require("globby"));
const path_1 = require("path");
const ramda_1 = require("ramda");
const webpack_merge_1 = require("webpack-merge");
const rules_2 = require("../rules");
async function default_1(presentationPath, config, options) {
    const layoutsConfig = await getLayoutsConfig(presentationPath, options);
    return (0, webpack_merge_1.mergeWithRules)({
        resolve: 'merge',
        module: {
            rules: {
                test: 'match',
                loader: 'match',
                options: 'replace',
            },
        },
    })(config, layoutsConfig);
}
exports.default = default_1;
;
async function getLayoutsConfig(presentationPath, options) {
    const { layout } = options;
    if (!layout) {
        return {};
    }
    const layoutPath = await getLayoutPath(presentationPath, layout);
    const systemSettings = await getSystemSettings(presentationPath, layoutPath);
    const [currentThemeName, blocksManifestPath, blocksThemesPath, rootComponentPath, themesManifestPath, themesPath,] = await Promise.all([
        getCurrentThemeName(presentationPath, layoutPath),
        ...await getResolvedPaths(presentationPath, layoutPath, systemSettings),
    ]);
    const resolvedSystemSettings = (0, ramda_1.mergeDeepRight)(systemSettings, {
        path: {
            blocksManifest: blocksManifestPath,
            blocksThemes: blocksThemesPath,
        },
    });
    const themesList = await (0, builder_utils_1.getThemesList)(presentationPath, themesManifestPath, themesPath);
    const currentBlocksList = (0, builder_utils_1.getCurrentBlocks)(presentationPath, resolvedSystemSettings);
    const uniqueBlocksList = (await (0, builder_utils_1.containsDynamicBlocksLoader)(presentationPath, rootComponentPath))
        ? currentBlocksList
        : [];
    const alias = await buildAliases(presentationPath, layoutPath);
    return {
        resolve: {
            alias,
        },
        module: {
            rules: [
                (0, rules_1.currentTheme)({
                    options: {
                        currentTheme: currentThemeName,
                        themes: themesList,
                    },
                }),
                (0, rules_1.currentBlocks)({
                    options: {
                        blocksList: uniqueBlocksList,
                    },
                }),
                (0, rules_2.entryEditor)({
                    options: {
                        blocksList: currentBlocksList,
                    },
                }),
            ],
        },
    };
}
;
async function buildAliases(presentationPath, layoutPath) {
    const layoutFilesList = await (0, globby_1.default)('**', {
        cwd: layoutPath,
        dot: true,
    });
    if (layoutFilesList.length) {
        return layoutFilesList.reduce((acc, file) => {
            acc[`${(0, path_1.resolve)(presentationPath, file)}`] = (0, path_1.resolve)(layoutPath, file);
            return acc;
        }, {});
    }
    console.log(`Layout ${layoutPath} not exists!`);
    process.exit(1);
}
async function getLayoutPath(presentationPath, layout) {
    const layoutsDirName = await (0, system_settings_1.getByPath)(presentationPath, 'path.layouts');
    return (0, path_1.join)(presentationPath, layoutsDirName, layout);
}
async function getCurrentThemeName(presentationPath, layoutPath) {
    try {
        return await (0, settings_1.getCurrentTheme)(layoutPath);
    }
    catch (_a) {
        return (0, settings_1.getCurrentTheme)(presentationPath);
    }
}
function getResolvedPaths(presentationPath, layoutPath, systemSettings) {
    const { blocksManifest, blocksThemes, rootComponent, themes, themesManifest, } = systemSettings.path;
    return Promise.all([
        getResolvedFilePath(presentationPath, layoutPath, blocksManifest),
        getResolvedFilePath(presentationPath, layoutPath, blocksThemes),
        getResolvedFilePath(presentationPath, layoutPath, rootComponent),
        getResolvedFilePath(presentationPath, layoutPath, themesManifest),
        getResolvedDirPath(presentationPath, layoutPath, themes),
    ]);
}
async function getSystemSettings(presentationPath, layoutPath) {
    return await (0, fs_extra_1.pathExists)((0, path_1.join)(layoutPath, '.ewizard/settings.json'))
        ? (0, system_settings_1.get)(layoutPath)
        : (0, system_settings_1.get)(presentationPath);
}
async function getResolvedFilePath(presentationPath, layoutPath, filePath) {
    const layoutFilePath = (0, path_1.join)(layoutPath, filePath);
    const presentationFilePath = (0, path_1.join)(presentationPath, filePath);
    return (await (0, fs_extra_1.pathExists)(layoutFilePath))
        ? layoutFilePath
        : presentationFilePath;
}
async function getResolvedDirPath(presentationPath, layoutPath, dirPath) {
    const layoutThemesPath = (0, path_1.join)(layoutPath, dirPath);
    const presentationThemesPath = (0, path_1.join)(presentationPath, dirPath);
    return await hasFiles(layoutThemesPath) ? layoutThemesPath : presentationThemesPath;
}
async function hasFiles(path) {
    const pattern = '**/*.{vue,js,css,scss,sass}';
    const existingFiles = await (0, globby_1.default)(pattern, { cwd: path });
    return !!existingFiles.length;
}
//# sourceMappingURL=webpack.config.js.map