"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DevBuilder = void 0;
const settings_1 = require("@ewizardjs/settings");
const system_settings_1 = require("@ewizardjs/system-settings");
const content_types_1 = require("@ewizardjs/content-types");
const path_1 = require("path");
const webpack_1 = __importDefault(require("webpack"));
const webpack_dev_server_1 = __importDefault(require("webpack-dev-server"));
const util_1 = require("util");
const find_free_port_1 = __importDefault(require("find-free-port"));
const builder_1 = require("./builder");
class DevBuilder extends builder_1.Builder {
    async build(options = {}) {
        const compiler = await this.getCompiler(options);
        return (0, util_1.promisify)(compiler.run).call(compiler);
    }
    async serve(options = {}) {
        const port = await this.findPort();
        const server = await this.createServer(options, port);
        server.start();
    }
    async createServer(options, port) {
        let configs = await Promise.all([
            this.getTemplatesConfig(Object.assign(Object.assign({}, options), { validate: false })),
            ...await this.getConfigWithComponentsListLibrary(Object.assign({ live: true }, options)),
        ]);
        if (options.layout) {
            const layoutConfig = await this.getLayoutConfig(options);
            configs = await this.getConfigWithComponentsListLibrary(options, layoutConfig);
        }
        const historyApiFallback = await this.getHistoryApiConfigurations();
        const serverConfig = {
            port,
            historyApiFallback,
            host: 'localhost',
            static: {
                directory: this.options.presentationPath,
                publicPath: '/',
            },
            hot: true,
            devMiddleware: {
                writeToDisk: (filePath) => !/hot-update\.(js|json)$/.test(filePath),
                stats: {
                    warningsFilter: stubSettingsDeclarationOutputWarnings,
                    modules: false,
                    hash: false,
                    children: false,
                    source: false,
                    timings: false,
                    builtAt: false,
                    version: false,
                    assets: false,
                    entrypoints: false,
                },
                publicPath: '/',
            },
            client: {
                logging: 'none',
                overlay: false,
            },
        };
        const server = new webpack_dev_server_1.default(serverConfig, (0, webpack_1.default)(configs));
        server.addAdditionalEntries((0, webpack_1.default)(configs).compilers[0]);
        return server;
    }
    async findPort() {
        const [port] = await (0, find_free_port_1.default)(3000);
        return port;
    }
    async getHistoryApiConfigurations() {
        const isEnabledHistoryApi = await this.isHistoryApiFallbackEnabled();
        const basePath = '/';
        if (!isEnabledHistoryApi) {
            return false;
        }
        return {
            rewrites: [
                {
                    from: /^\/.*/,
                    to: ({ request, parsedUrl }) => {
                        var _a;
                        const referer = (_a = request === null || request === void 0 ? void 0 : request.headers) === null || _a === void 0 ? void 0 : _a.referer;
                        const { pathname: resourcePathName } = parsedUrl;
                        if (!referer || !resourcePathName || isNotResourcePath(resourcePathName)) {
                            return basePath;
                        }
                        const { pathname: urlPathName } = new URL(referer);
                        const pathName = urlPathName.replace(/[^\/]*$/, '');
                        return resourcePathName.replace(pathName, '/');
                    },
                },
            ],
        };
    }
    async isHistoryApiFallbackEnabled() {
        const [isSite, isHistoryMode] = await Promise.all([
            this.isSiteContentType(),
            this.isHistoryRouterMode(),
        ]);
        return isSite && isHistoryMode;
    }
    async isSiteContentType() {
        const { content } = await (0, system_settings_1.get)(this.options.presentationPath);
        const { type, subtype } = content;
        return type === content_types_1.ContentTypes.SITE || subtype === content_types_1.ContentTypes.SITE;
    }
    async isHistoryRouterMode() {
        var _a;
        const { navigation } = await (0, settings_1.getRaw)(this.options.presentationPath);
        return ((_a = navigation === null || navigation === void 0 ? void 0 : navigation.router) === null || _a === void 0 ? void 0 : _a.mode) === 'history';
    }
}
exports.DevBuilder = DevBuilder;
function isNotResourcePath(resourcePath) {
    return !(0, path_1.extname)(resourcePath);
}
function stubSettingsDeclarationWarnings(error) {
    const settingsDeclarations = /node_modules\/(@ewizardjs\/)?settings\/.*\.d\.ts/;
    return !Boolean(settingsDeclarations.test(typeof error === 'string' ? error : error.message));
}
function stubSettingsDeclarationOutputWarnings(warning) {
    return !stubSettingsDeclarationWarnings(warning);
}
//# sourceMappingURL=devBuilder.js.map